'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var error = require('ipfs-message-port-protocol/error');

class Query {
  constructor(namespace, method, input) {
    this.result = new Promise((resolve, reject) => {
      this.succeed = resolve;
      this.fail = reject;
      this.namespace = namespace;
      this.method = method;
      this.input = input;
      this.abortController = new AbortController();
      this.signal = this.abortController.signal;
    });
  }
  abort() {
    this.abortController.abort();
    this.fail(new AbortError());
  }
}
class Server {
  constructor(services) {
    this.services = services;
    this.queries = Object.create(null);
  }
  connect(port) {
    port.addEventListener('message', this);
    port.start();
  }
  disconnect(port) {
    port.removeEventListener('message', this);
    port.close();
  }
  handleEvent(event) {
    const data = event.data;
    switch (data.type) {
    case 'query': {
        this.handleQuery(data.id, new Query(data.namespace, data.method, data.input), event.target);
        return undefined;
      }
    case 'abort': {
        return this.abort(data.id);
      }
    default: {
        throw new UnsupportedMessageError(event);
      }
    }
  }
  abort(id) {
    const query = this.queries[id];
    if (query) {
      delete this.queries[id];
      query.abort();
    }
  }
  async handleQuery(id, query, port) {
    this.queries[id] = query;
    await this.run(query);
    delete this.queries[id];
    if (!query.signal.aborted) {
      try {
        const value = await query.result;
        const transfer = value.transfer;
        delete value.transfer;
        port.postMessage({
          type: 'result',
          id,
          result: {
            ok: true,
            value
          }
        }, transfer);
      } catch (error$1) {
        port.postMessage({
          type: 'result',
          id,
          result: {
            ok: false,
            error: error.encodeError(error$1)
          }
        });
      }
    }
  }
  run(query) {
    const {services} = this;
    const {namespace, method} = query;
    const service = services[namespace];
    if (service) {
      if (typeof service[method] === 'function') {
        try {
          const result = service[method]({
            ...query.input,
            signal: query.signal
          });
          Promise.resolve(result).then(query.succeed, query.fail);
        } catch (error) {
          query.fail(error);
        }
      } else {
        query.fail(new RangeError(`Method '${ method }' is not found`));
      }
    } else {
      query.fail(new RangeError(`Namespace '${ namespace }' is not found`));
    }
  }
  execute(data) {
    const query = new Query(data.namespace, data.method, data.input);
    this.run(query);
    return query.result;
  }
}
class UnsupportedMessageError extends RangeError {
  constructor(event) {
    super('Unexpected message was received by the server');
    this.event = event;
  }
  get name() {
    return this.constructor.name;
  }
}
const AbortError = class AbortError extends Error {
  get name() {
    return this.constructor.name;
  }
};

exports.AbortError = AbortError;
exports.Query = Query;
exports.Server = Server;
exports.UnsupportedMessageError = UnsupportedMessageError;
